#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

################################################################################
# Created       : Martin Foster
# Modified      : 05-Sep-2006
################################################################################
#
# Registration - Script part of Ethereal Realms designed to register patrons 
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3456 Wolfe Cres
#                 Halifax, Nova-Scotia
#                 B3L 3S2
#
#################################################################################

use CGI;							# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Digest::MD5 qw(md5_hex);					# MD5 digest cryptographic algorithm
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Filter;						# Filter handler
use Ethereal::Geo;						# Geographical location finder
use Ethereal::Mail;						# Mail handling
use Ethereal::Option;						# Option handling
use Ethereal::Param;						# Parameter control
use Ethereal::Template;						# Template handler

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 50;					# Maximum posts
$CGI::DISABLE_UPLOADS = 1;					# Disable uploads

#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $geo;							# Geographical data
my $send;							# Mail name
my $param;							# Parameter hash
my $tmpl;							# Template management

my $capture;							# Arbitrary captured line
my $hostname;							# Remote host hostname
my $scriptname;							# Name of called script

my $city;							# City
my $country;							# Country
my $locate;							# Combined field
my $region;							# Region

my %config;							# Login hash
my %system;							# System hash


#################################################################################
# Program Area
#################################################################################

	
	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();

	# Establish connection
	$database->Connect($cgi);

	# Parameter handling
	$param = new Ethereal::Param($database, $cgi);


	# Retreive hostname
	$hostname = $cgi->remote_host();


	# Retreive hash
	$database->GetHashUser(\%config);

	# template Management
	$tmpl = new Ethereal::Template(\%config);


	# Entrance without parameters
	unless ($cgi->param())
	{
		# Pull scriptname
		$scriptname = $cgi->url(-relative=>1);


		# Headers
		print $cgi->header();
		print $database->DocumentGetHeader(), "\n";

		# Display initial form
		if ($scriptname =~ /^confirmation/i)
		{
			# Confirmation form
			InformationConf($database, $cgi, $param, $tmpl, \%config);
		}
		else
		{
			# Registration form
			InformationFormInit($database, $cgi, $param, $tmpl, \%config);
		}


		# Footer
		print $database->DocumentGetFooter(), "\n";
	}

	# Entrace with parameters
	else
	{
		# Retreive parameters
		$param->GetParam();

		# If no registration codes or logins		
		unless ((defined($param->{'LOGIN'}))
		     && (defined($param->{'CONFCODE'})))
		{
			# Headers
			print $cgi->header();
			print $database->DocumentGetHeader(), "\n";

			# If not accepted show AUP
			unless (defined($param->{'ACCEPT'}))
			{ 
				# Check contrains if correct then print AUP
				PolicyForm($database, $cgi, $param, $tmpl, \%config)
				  if ConstraintHandler($database, $cgi, $param, $tmpl, \%config);
			}

			# If accepted process request
			elsif (defined($param->{'ACCEPT'}))
			{
				# Record information
				RequestHandler($database, $cgi, $param, \%config);

				# Send to confirmation screen
				InformationConf($database, $cgi, $param, $tmpl, \%config);
			}

			# Footer
			print $database->DocumentGetFooter(), "\n";
		}

		# If such a code was sent
		else
		{ 
			# Check if registration is valid
			if (AcceptHandler($database, $param))
			{
				# Geographical information
				# Create instance
				$geo = new Ethereal::Geo($database);

				# Pull the information
				($country,
				 $region,
				 $city) = $geo->Locate($hostname);

			 	# Was there a find?
				$locate = (defined($country))
				  ? '(' . $city . ',' . $region . ',' . $country . ')'
				  : $config{'TxtNil'};


				# HTML Display
				# Headers
				print $cgi->header();
				print $database->DocumentGetHeader(), "\n";

				# Display fact they were accepted
				$tmpl->Show('TmplRegAccepted');

				# Footer
				print $database->DocumentGetFooter(), "\n";


				# Email notification
				# Retreive Hash
				$database->GetHashSystem(\%system);

				# Retreive information
				my ($verified) = $database->DocumentGetConfirmed();
				my ($email)    = $database->DataGetPuppeteerEmail($param->{'LOGIN'});
				my ($name)     = $database->DataGetPuppeteerName($param->{'LOGIN'});


				# Mail sending
				# Initial handle
				$send = tie(*MAIL, 'Ethereal::Mail', $database);
				
				# Setup
				$send->SetSubject("Welcome $name ($email) to $system{SetInfoServer}");
				$send->SetSearch(
					EMAIL      => $email,
					FULLNAME   => $name,
					HOSTNAME   => $hostname . ' ' . $locate,
					LOGINNAME  => $param->{'LOGIN'},
					PORTAL     => $system{'LocScriptPortal'},
					SERVERNAME => $system{'SetInfoServer'}
				 );

				# Recipients
				$send->AddTo("$name <$email>");
				$send->AddBCC("$system{SetSubmissionName} <$system{SetSubmissionAddress}>");

				# Message
				print MAIL $verified;

				# Close/Send
				close(MAIL);
			}
			
			# If not redirect to portal
			else
			{ print $cgi->redirect($config{'LnkPortal'}); }

		}
	}


################################################################################
# Sub-Routines
################################################################################

#####################
# Acceptence Handler
#
# Will determine is registration code is correct.  If so, the script will remove
# registration information allowing user to login as needed.

sub AcceptHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter list

	my $choice;						# Captured data

	#####################
	# Program area

	# Retreive login
	($choice) = $database->DataGet("SELECT PuppeteerLogin
		FROM  PuppeteerRegistration
		WHERE PuppeteerLogin=?
		  AND PuppeteerConfCode=?",
	 $param->{'LOGIN'},
	 $param->{'CONFCODE'}
	);
	
	# If entry was found
	if (defined($choice))
	{
		# Erase lock
		$database->Quick("DELETE 
			FROM PuppeteerRegistration 
			WHERE PuppeteerLogin=? 
			  AND PuppeteerConfCode=?", 

		 $param->{'LOGIN'}, 
		 $param->{'CONFCODE'}
		);

		# Return true
		return 1;
	}

	return 0;
}


#####################
# Check Parameter
# 
# Checks parameters and verifies if data exists already in table or if they
# fit criteria.

sub CheckParam
{
	#####################
	# Data members
	my $subject = shift; 					# Arbitrary retrieval
	my $choice  = shift;					# Choice on the matter
	my $check   = shift;					# Check value
	my $user    = shift;					# Login name

	my $filter;						# Filtering ability

	my $handle;						# Handle name
	my $short;						# Short name

	#####################
	# Program area

	# Lenght
	if ($choice == 0)
	{
		if (length($subject) < 3)
		{ return 0; }					# False if password short

		return 1;					# Return true if 5 or more
	}

	# E-Mail
	if ($choice == 1)
	{
		# Basic verification
		return 0 if ($check !~ /^\w(\w|\.|\-)*?\@(\w|\-)*?\..*?/s);

		# Retreive Email
		($choice) = $subject->DataGet("SELECT PuppeteerEmail
			FROM  Puppeteer
			WHERE PuppeteerEmail ILIKE ?", $check);

		# Determin $choice and return
		$choice = (defined($choice)) ? 0 : 1;
		return $choice;
	}

	# Login name availability
	elsif ($choice == 2)
	{
		# Basic verification
		return 0 if ($check !~ /^\w+$/s);
		
		# Retreive Name
		($choice) = $subject->DataGetPuppeteerName($check);

		# Determin $choice and return
		$choice = (defined($choice)) ? 0 : 1;

		# Returns decision
		return $choice;
	}

	# Puppet name availability
	elsif ($choice == 3)
	{
		# Create instance
		$filter = new Ethereal::Filter();
		
		# Strip HTML
		$handle =  $filter->StripHTML($check);

		# Additional filtering
		$handle =~ s/~//gs;

		# Generate short name
		$short  =  lc($handle);
		$short  =~ s/-/_/gs;
		$short  =~ s/\s/_/gs;
		$short  =~ s/\W//gs;

		# Retreive information
		($choice) = $database->DataGetPuppetCreate($handle, $short, $user);

		# Determin $choice and return
		$choice = (defined($choice)) ? 0 : 1;

		# Returns decision
		return $choice;
	}
}

# Alias for Lenght
sub CheckParamLenght
{ return CheckParam(shift, 0); }

# Alias for Email
sub CheckParamEmail
{ return CheckParam(shift, 1, shift); }

# Alias for Login
sub CheckParamLogin
{ return CheckParam(shift, 2, shift); }

# Alias for Login
sub CheckParamHandle
{ return CheckParam(shift, 3, shift); }


#####################
# Constraint Handler
#
# Checks contraints, if they are correct returns true, otherwise requests
# repost of data missing or off.

sub ConstraintHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter reference
	my $tmpl     = shift;					# Template
	my $config   = shift;					# Configurations


	#####################
	# Program area

	# Username/login verfication
	unless ((CheckParamLogin($database, $param->{'USERNAME'})) 
	 && (CheckParamLenght($param->{'USERNAME'})))
	{
		# Login information failed
		InformationFormLogin($database, $cgi, $param, $tmpl, $config); 
		
		# Return false
		return 0;
	}

	# Username/login verfication
	unless ((CheckParamHandle($database, $param->{'CHARNAME'}, $param->{'USERNAME'})) 
	 && (CheckParamLenght($param->{'CHARNAME'})))
	{
		# Login information failed
		InformationFormHandle($database, $cgi, $param, $tmpl, $config); 
		
		# Return false
		return 0;
	}

	# Full name verification
	unless (CheckParamLenght($param->{'FULLNAME'}))
	{
		# Full name failed
		InformationFormFullname($database, $cgi, $param, $tmpl, $config);
		
		# Return false
		return 0;
	}

	# EMail check
	unless (($param->{'EMAIL'} eq $param->{'EMAILTWICE'}) 
	 && (CheckParamLenght($param->{'EMAIL'})) 
	 && (CheckParamEmail($database, $param->{'EMAIL'})))
	{
		# Email failed
		InformationFormEmail($database, $cgi, $param, $tmpl, $config);
		
		# Return false
		return 0;
	}

	# Password verification
	unless (($param->{'PASSONCE'} eq $param->{'PASSTWICE'}) 
	 && (CheckParamLenght($param->{'PASSONCE'})))
	{
		# Password failed
		InformationFormPassword($database, $cgi, $param, $tmpl, $config);
		
		# Return false
		return 0;
	}

	return 1;
}


#####################
# Information Conf
#
# Information form for confirmation purposes.   This is called when you reach
# a link with confirmation in the start of it's name.  Will essentially allow
# one to bypass bad Email links.

sub InformationConf
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handling
	my $tmpl     = shift;					# Template handling
	my $config   = shift;					# Configuration hash

	#####################
	# Program area

	# Start form
	print $cgi->start_form();

	# Display template
	$tmpl->Show('TmplRegConf',
		WLOGIN => $cgi->textfield('LOGIN', '', 10, 10),
		WCONFC => $cgi->textfield('CONFCODE', '', 10, 10),
		WSBMIT => $cgi->submit($config->{'TxtConfirm'})
	 );
	
	# End form
	print $cgi->end_form();
}

#####################
# Information Form
#
# Initially displayed sub-routine, for sole purpose of gathering infromation.

sub InformationForm
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handling
	my $tmpl     = shift;					# Template handler
	my $config   = shift;					# Confirmation form
	my $choice   = shift;					# Choice of the matter

	my $option;						# Option handler

	my $message;						# Message handling

	my @javascript;						# Javascript prefs array
	my @chatpref;						# Chat prefs array
	my @formatting;						# Formatting options

	my %system;						# System hash


	#####################
	# Program area

	# Create instance
	$option = new Ethereal::Option();

	# Link to hashes
	$database->GetHashSystem(\%system);


	# Retreive arrays with necessary options
	$option->Split(\@javascript, $config->{'OptJavascript'});
	$option->Split(\@chatpref, $config->{'OptChatPref'});
	$option->Split(\@formatting, $config->{'OptFormat'});


	# Invalid sections
	$message = $config->{'ErrUsername'} if ($choice == 0);
	$message = $config->{'ErrFullName'} if ($choice == 1);
	$message = $config->{'ErrEmail'}    if ($choice == 2);
	$message = $config->{'ErrPassword'} if ($choice == 3);
	$message = $config->{'ErrHandle'}   if ($choice == 4);

	# Sanity checks
	$param->{'USERNAME'} = '' unless ((defined($param->{'USERNAME'})) && ($choice != 0));
	$param->{'FULLNAME'} = '' unless ((defined($param->{'FULLNAME'})) && ($choice != 1));
	$param->{'EMAIL'}    = '' unless ((defined($param->{'EMAIL'}))    && ($choice != 2));
	$param->{'PASSONCE'} = '' unless ((defined($param->{'PASSONCE'})) && ($choice != 3));
	$param->{'CHARNAME'} = '' unless ((defined($param->{'CHARNAME'})) && ($choice != 4));


	# Generate widgets
	my $username = $cgi->textfield('USERNAME', $param->{'USERNAME'}, 10, 10);
	my $fullname = $cgi->textfield('FULLNAME', $param->{'FULLNAME'}, 20, 30);
	my $charname = $cgi->textfield('CHARNAME', $param->{'CHARNAME'}, 20, 30);
	my $emailone = $cgi->textfield('EMAIL', $param->{'EMAIL'}, 20, 45);
        my $emailtwo = $cgi->textfield('EMAILTWICE', $param->{'EMAIL'}, 20, 45);
	my $upassone = $cgi->password_field('PASSONCE', $param->{'PASSONCE'}, 10, 15);
	my $upasstwo = $cgi->password_field('PASSTWICE', $param->{'PASSONCE'}, 10, 15);

	# Initialize Javascript and Chat preferences
	my $javascript = $cgi->radio_group('JAVASCRIPT', \@javascript, $javascript[0],'true');
	my $chatpref   = $cgi->radio_group('CHATPREFS', \@chatpref, $chatpref[2],'true');
	my $formatting = $cgi->radio_group('FORMATPREFS', \@formatting, $formatting[0],'true');

	# Additional widgets
	my $submit  = $cgi->submit($config->{'TxtRegister'});
	my $reset   = $cgi->reset($config->{'TxtReset'});


	# Display HTML
	# Begin form
	print $cgi->start_form();
	
	# Display template
	$tmpl->Show('TmplRegGather',
		MERRORMSG => $message,
		WUSERNAME => $username,
		WFULLNAME => $fullname,
		WCHARNAME => $charname,
		WEMAILONE => $emailone,
	        WEMAILTWO => $emailtwo,
		WUPASSONE => $upassone,
		WUPASSTWO => $upasstwo,
		WRADJAVAS => $javascript,
		WRADCHATP => $chatpref,
		WRADFORMT => $formatting,
		WSUBMIT   => $submit,
		WRESET    => $reset
	 );	

	# End form
	print $cgi->end_form(), "\n";
}

# Alias for Initilization
sub InformationFormInit
{ InformationForm(shift, shift, shift, shift, shift, 99); }

# Alias for Login
sub InformationFormLogin
{ InformationForm(shift, shift, shift, shift, shift, 0); }

# Alias for Fullname
sub InformationFormFullname
{ InformationForm(shift, shift, shift ,shift, shift, 1); }

# Alias for EMail
sub InformationFormEmail
{ InformationForm(shift, shift, shift ,shift, shift, 2); }

# Alias for pasword
sub InformationFormPassword
{ InformationForm(shift, shift, shift, shift, shift, 3); }

# Alias for Login
sub InformationFormHandle
{ InformationForm(shift, shift, shift, shift, shift, 4); }


#####################
# Policy Form
#
# Displays the acceptable use policy flag and embeds all information gathered.
# Also adds a tag to mark acceptance.

sub PolicyForm
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter
	my $tmpl     = shift;					# Template handler
	my $config   = shift;					# Configuration hash


	#####################
	# Program area

	# HTML
	# Form display
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), ACCEPT=>'True'), "\n";

	# Display
	$tmpl->Show('TmplRegPolicy',
		WACCEPT => $cgi->submit($config->{'TxtAccept'})
	 );

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Request Handler
#
# Takes in the parameters, sending a part of the information to the database, then
# sending a confirmation E-Mail to the poster.

sub RequestHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Common Gateway Interface
	my $param    = shift;					# Parameter hash
	my $config   = shift;					# Configuration hash

	my $confcode;						# Confirmation code
	my $inline;						# Authenthication string
	my $invitation;						# Invitation	

	my $handle;						# Handle name
	my $short;						# Short name

	my $option;						# Options handler
	my $send;						# Mail sender

	my %system;						# System hash to DBM


	#####################
	# Program area

	# Create instance
	$option   = new Ethereal::Option();

	# Link to hashes
	$database->GetHashSystem(\%system);

	# Information on registration
	$invitation = $database->DocumentGetInvitation();

	# Confirmation code
	($confcode) = $database->DataGet("SELECT PuppeteerConfCode
		FROM PuppeteerRegistration
		WHERE PuppeteerLogin=?",

	 $param->{'USERNAME'}
	);

	# Already exists
	unless (defined($confcode))
	{
		# Generate new code
		$confcode = md5_hex(rand());
		$confcode = substr($confcode, 0, 10);
	}


	# Create authenthication string
	$inline = $param->Crypt(LOGIN=>$param->{'USERNAME'}, CONFCODE=>$confcode);


	# Evaluate
	eval
	{
		# Mail sending
		# Initiate handle
		$send = tie(*MAIL, 'Ethereal::Mail', $database);

		# Initial setup
		$send->SetSubject("An invitation to $system{SetInfoServer}");
		$send->SetSearch(
			AUTHSTRING  => "$system{LocScriptConf}$inline",
			CONFADDR    => $system{'LocScriptConf'},
			CONFCODE    => $confcode,
			FULLNAME    => $param->{'FULLNAME'},
			LOGINNAME   => $param->{'USERNAME'},
			PORTAL      => $system{'LocScriptPortal'},
			RETURNEMAIL => $system{'SetInfoContactAddress'},
			SERVERNAME  => $system{'SetInfoServer'}
		 );

		# Recipients
		$send->AddTo("$param->{'FULLNAME'} <$param->{EMAIL}>");

		# Message
		print MAIL $invitation;

		# Close and send 
		close(MAIL);
	

		# Puppeteers
		# Insert Puppeteer
		$database->Write("INSERT INTO Puppeteer 
			(PuppeteerLogin,
			 PuppeteerPassword,
			 PuppeteerName,
			 PuppeteerEMail,
			 PuppeteerFormatting,
			 PuppeteerChatPref,
			 PuppeteerJavascript,
			 PuppeteerAdminPrivs,
			 PuppeteerSuperPrivs,
			 PuppeteerModPrivs)
			VALUES (?,?,?,?,?,?,?,'no','no','no')", 

  		  $param->{'USERNAME'}, 
		  crypt(substr($param->{'PASSONCE'}, 0, 8), $confcode), 
		  $param->{'FULLNAME'}, 
		  $param->{'EMAIL'}, 
		  $option->FindFormatting($config->{'OptFormat'},     $param->{'FORMATPREFS'}),
		  $option->FindChatPrefs($config->{'OptChatPref'},    $param->{'CHATPREFS'}),
		  $option->FindJavascript($config->{'OptJavascript'}, $param->{'JAVASCRIPT'})
		);

		# Registration codes
		$database->Write("INSERT INTO PuppeteerRegistration 
			(PuppeteerLogin,
			 PuppeteerConfCode)
			VALUES (?,?)", 

		 $param->{'USERNAME'},
		 $confcode
		);


		# Puppets
		$handle = $param->{'CHARNAME'};

		# Generate short name
		$short  =  lc($handle);
		$short  =~ s/-/_/gs;
		$short  =~ s/\s/_/gs;
		$short  =~ s/\W//gs;

		
		# Create Puppet
		$database->Write("INSERT INTO Puppet
			(PuppetName,
			 PuppeteerLogin,
			 PuppetShort,
			 PuppetDefault)
			VALUES (?,?,?,'yes')",

		 $handle,
		 $param->{'USERNAME'},
		 $short
		);

		# Commit
		$database->Commit();
	};

	# If block failed
	if ($@)
	{
		# Issue warning
		warn("Transaction aborted: $@");

		# Undo incomplete changes
		$database->Rollback();
	}
}
